# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import yaml
import logging

from dput.changes import logger, parse_changes_file
log = logging.getLogger()
logger.setLevel(log.level)

from ci_utils.data_store import DataStore

auth = os.path.join(os.environ["HOME"], '.ubuntu-ci/auth_config.yaml')
try:
    with open(auth) as fp:
        auth_config = yaml.safe_load(fp)
except IOError, e:
    sys.exit("Missing authentication info. " +
             "Add 'auth_url', 'auth_user', 'auth_password', " +
             "and 'auth_tenant_name' to {}".format(auth))


class FileToUploadNotFound(IOError):
    """Raised if a file mentioned in .changes is not found."""


class UploadDirNotFound(IOError):
    """Raised if the provided upload directory is not found."""


class ChangesFileNotFound(IOError):
    """Raised if .changes file is not found."""


class ChangesProcessor:
    """Class to parse, validate and upload source files to the datastore."""

    def __init__(self, changes_filepath, files_to_upload_dir=None):
        # Needs to be provided with the full path to the changes file.
        if not os.path.exists(changes_filepath):
            raise ChangesFileNotFound(changes_filepath)
        self.changes_filepath = changes_filepath

        if files_to_upload_dir is not None:
            if not os.path.exists(files_to_upload_dir):
                raise UploadDirNotFound(files_to_upload_dir)
            self.files_to_upload_dir = files_to_upload_dir
        else:
            self.files_to_upload_dir = os.path.dirname(changes_filepath)
        self.files_to_upload = []
        self.changes = None

    @property
    def source_package_name(self):
        return self.changes._data["Source"]

    @property
    def source_version(self):
        return self.changes._data["Version"]

    def validate(self, check_signature=False):
        changes = self.changes
        try:
            # We don't care if signatures fail for now
            changes.validate(check_signature=check_signature)
        except Exception, exc:
            raise exc

    def parse(self):
        try:
            self.changes = parse_changes_file(
                filename=self.changes_filepath,
                directory=self.files_to_upload_dir)
            # Check if all files to upload mentioned by .changes can be found
            # at the provided directory. If not, abort the upload.
            for file_ in self.changes.get_files():
                if os.path.exists(file_):
                    self.files_to_upload.append(file_)
                else:
                    raise FileToUploadNotFound(file_)
            self.files_to_upload.append(self.changes.get_changes_file())
        except Exception, exc:
            raise exc

    def process(self, check_signature=False):
        self.parse()
        self.validate(check_signature=check_signature)
        return self.files_to_upload


def upload_files(file_path, ticket_id):
    ds = DataStore('ticket-system.{}'.format(ticket_id), auth_config)
    with open(file_path, 'r') as fp:
        location = ds.put_file(file_path, fp)
    return location
