#!/usr/bin/python

import mock
import unittest
import os
import sys
import tempfile
import shutil

sys.path.append(os.path.abspath(os.path.dirname(sys.argv[0])))
import deploy


class TestDeploy(unittest.TestCase):

    def test_check_environment(self):
        # If we haven't sourced a novarc, this program should bail out.
        with mock.patch.dict('os.environ', {'OS_USERNAME': ''}):
            with self.assertRaises(SystemExit):
                deploy.check_environment()

    def test_check_dependencies(self):
        # If we cannot find any of our dependencies, this program should bail
        # out.
        with mock.patch.dict('os.environ', {'PATH': ''}):
            with self.assertRaises(SystemExit):
                deploy.check_dependencies()

    def test_generate_empty_list(self):
        working = tempfile.mkdtemp()
        self.addCleanup(lambda: shutil.rmtree(working))

        # Calling without anything in the directory should produce an empty
        # list.
        self.assertEqual(list(deploy.generate_argument_list(working)), [])

    def test_generate_explicit_list(self):
        working = tempfile.mkdtemp()
        self.addCleanup(lambda: shutil.rmtree(working))

        expected = os.path.join(working, 'foo.yaml')
        with open(expected, 'w') as fp:
            fp.write('nothing')
        # Calling with a basic configuration should produce the full path to
        # that configuration in the form ('-c', path).
        gen = deploy.generate_argument_list(working)
        self.assertEqual([('-c', expected)], list(gen))

    def test_populate_templates_empty_list(self):
        fake_deployer_dir = tempfile.mkdtemp()
        self.addCleanup(lambda: shutil.rmtree(fake_deployer_dir))
        working = tempfile.mkdtemp()
        self.addCleanup(lambda: shutil.rmtree(working))

        deploy.populate_templates(fake_deployer_dir, working)
        # Nothing happened
        self.assertEqual([], os.listdir(working))

    def test_populate_templates_explicit_list(self):
        fake_deployer_dir = tempfile.mkdtemp()
        self.addCleanup(lambda: shutil.rmtree(fake_deployer_dir))
        working = tempfile.mkdtemp()
        self.addCleanup(lambda: shutil.rmtree(working))

        expected = os.path.join(fake_deployer_dir, 'config.yaml.tmpl')
        with open(expected, 'w') as fp:
            fp.write('me: ${OS_USERNAME}')
        deploy.populate_templates(fake_deployer_dir, working)
        self.assertEqual(['config.yaml'], os.listdir(working))
        with open(os.path.join(working, 'config.yaml')) as expanded:
            content = expanded.read()
        self.assertEqual('me: {}'.format(os.environ['OS_USERNAME']), content)


if __name__ == '__main__':
    unittest.main()
