# Ubuntu CI Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import mock
import os
import sys
import unittest

sys.path.append(os.path.join(os.path.dirname(__file__), '../../bin'))
import lander_archiver
sys.path.append(os.path.join(os.path.dirname(__file__), '../../ci-utils'))
from ci_utils import data_store


class TestArchiver(unittest.TestCase):
    '''Tests for the lander_archiver.py script.'''

    def setUp(self):
        super(TestArchiver, self).setUp()
        self.archiver = lander_archiver.LanderArchiver()

    def test_get_api_json_url(self):
        '''Verify correct expansion of the jenkins api json path.'''
        url = 'http://jenkins:8080/job/lander_master'
        self.assertEqual(url + '/api/json',
                         self.archiver.get_api_json_url(url))

    def test_get_console_log_url(self):
        '''Verify correct expansion of the jenkins console log path.'''
        url = 'http://jenkins:8080/job/lander_master'
        self.assertEqual(url + '/consoleText',
                         self.archiver.get_console_log_url(url))

    def test_get_artifact_url(self):
        '''Verify correct expansion of the jenkins artifact path.'''
        url = 'http://jenkins:8080/job/lander_master'
        name = 'artifact.1'
        self.assertEqual('{}/{}/{}'.format(url, 'artifact', name),
                         self.archiver.get_artifact_url(url, name))

    @mock.patch('lander_archiver.urllib2.Request')
    def test_get_url_contents(self, request_mock):
        '''Verify generation of EnvironmentError on a failed url load.'''
        request_mock.side_effect = IOError
        self.assertRaises(
            EnvironmentError,
            self.archiver.get_url_contents,
            'http://jenkins:8080/bad-url/')

    def test_get_datastore_config(self):
        '''Verify creation of a datastore configuration.'''
        config = {'username', 'user',
                  'password', 'pass'}
        self.archiver.get_datastore_config_contents = lambda x: config
        self.assertEqual(config,
                         self.archiver.get_datastore_config('filename'))

    def test_get_datastore_config_exception(self):
        '''Verify an error is logged on failure to load a datastore config.'''
        get_mock = mock.Mock()
        get_mock.side_effect = IOError
        self.archiver.get_datastore_config_contents = get_mock
        log_mock = mock.Mock()
        self.archiver.log = mock.Mock()
        self.archiver.log.error = log_mock
        self.assertIsNone(self.archiver.get_datastore_config('filename'))
        log_mock.assert_called_with('Failed to load datastore configuration: ')

    @mock.patch('lander_archiver.data_store.DataStore')
    def test_get_datastore(self, data_store_mock):
        '''Verify a data store is returned assuming mocked sources.'''
        datastore = 'Mocked datastore'
        request_id = 'request-1'
        data_store_mock.return_value = datastore
        self.assertEqual(datastore,
                         self.archiver.get_datastore(request_id, {}))

    @mock.patch('lander_archiver.data_store.DataStore')
    def test_get_datastore_exception(self, data_store_mock):
        '''Verify a data store is returned assuming mocked sources.'''
        request_id = 'request-1'
        data_store_mock.side_effect = data_store.DataStoreException
        log_mock = mock.Mock()
        self.archiver.log = mock.Mock()
        self.archiver.log.error = log_mock
        self.assertIsNone(self.archiver.get_datastore(request_id, {}))
        log_mock.assert_called_with('Failed to create datastore: ')

    def test_get_artifact_list(self):
        '''Verify an artifact list is returned from jenkins json data.'''
        artifact_list = [
            'http://jenkins:8080/path/to/artifact.1',
            'http://jenkins:8080/path/to/artifact.2']
        jenkins_json = {'artifacts':
                        [{'relativePath': x} for x in artifact_list]}
        self.archiver.get_url_contents = lambda x: json.dumps(jenkins_json)
        self.assertEqual(artifact_list,
                         self.archiver.get_artifact_list('url'))

    def test_get_artifact_list_none(self):
        '''Verify that a job with no artifacts returns an empty list.'''
        artifact_list = []
        jenkins_json = {'artifacts': artifact_list}
        self.archiver.get_url_contents = lambda x: json.dumps(jenkins_json)
        self.assertEqual(artifact_list,
                         self.archiver.get_artifact_list('url'))

    def test_store_artifact(self):
        '''Verify that storing an artifact results in a location.'''
        location = 'http://data-store/bucket/file'
        datastore = mock.Mock()
        datastore.put_file = mock.Mock()
        datastore.put_file.return_value = location
        self.archiver.get_url_contents = lambda x: 'data'
        self.assertEqual(
            location, self.archiver.store_artifact(datastore, 'name', 'url'))

    def test_store_artifact_error(self):
        '''Verify that a datastore error causes a log message.'''
        datastore = mock.Mock()
        datastore.put_file = mock.Mock()
        datastore.put_file.side_effect = data_store.DataStoreException
        log_mock = mock.Mock()
        self.archiver.log = mock.Mock()
        self.archiver.log.error = log_mock
        self.archiver.get_url_contents = lambda x: 'data'
        self.assertIsNone(
            self.archiver.store_artifact(datastore, 'name', 'url'))
        log_mock.assert_called_with('Failed to archive artifact [name]: ')

    def test_archive_console_log(self):
        '''Verify archiving of the console log.'''
        location = 'http://data-store/bucket/file'
        datastore = mock.Mock()
        datastore.put_file = mock.Mock()
        datastore.put_file.return_value = location
        log_mock = mock.Mock()
        self.archiver.log = mock.Mock()
        self.archiver.log.info = log_mock
        self.archiver.get_url_contents = lambda x: 'console data'
        self.archiver.archive_console_log(datastore, 'master',
                                          'http://jenkins:8080/job/master/')
        log_mock.assert_called_with('Console log: {}'.format(location))

    @mock.patch('lander_archiver.urllib2.Request')
    def test_archive_console_log_error(self, request_mock):
        '''Verify a console log load error raises an exception.'''
        request_mock.side_effect = IOError
        location = 'http://data-store/bucket/file'
        datastore = mock.Mock()
        datastore.put_file = mock.Mock()
        datastore.put_file.return_value = location
        log_mock = mock.Mock()
        self.archiver.log = mock.Mock()
        self.archiver.log.info = log_mock
        with self.assertRaises(EnvironmentError):
            self.archiver.archive_console_log(
                datastore, 'master', 'http://jenkins:8080/job/master/')
        self.assertEqual(0, log_mock.call_count)

    def test_archive_artifacts(self):
        '''Verify archiving of an artifact.'''
        location = 'http://data-store/bucket/file'
        datastore = mock.Mock()
        datastore.put_file = mock.Mock()
        datastore.put_file.return_value = location
        log_mock = mock.Mock()
        self.archiver.log = mock.Mock()
        self.archiver.log.info = log_mock
        artifact_list = [
            'http://jenkins:8080/path/to/artifact.1']
        jenkins_json = {'artifacts':
                        [{'relativePath': x} for x in artifact_list]}
        self.archiver.get_url_contents = lambda x: json.dumps(jenkins_json)
        self.archiver.archive_artifacts(datastore, 'master',
                                        'http://jenkins:8080/job/master/')
        for artifact in artifact_list:
            log_mock.assert_called_with('Artifact [{}]: '
                                        '{}'.format(artifact, location))

    @mock.patch('lander_archiver.urllib2.Request')
    def test_archive_artifacts_error(self, request_mock):
        '''Verify an artifact load error raises an exception.'''
        request_mock.side_effect = IOError
        location = 'http://data-store/bucket/file'
        datastore = mock.Mock()
        datastore.put_file = mock.Mock()
        datastore.put_file.return_value = location
        log_mock = mock.Mock()
        self.archiver.log = mock.Mock()
        self.archiver.log.info = log_mock
        with self.assertRaises(EnvironmentError):
            self.archiver.archive_artifacts(datastore, 'master',
                                            'http://jenkins:8080/job/master/')
        self.assertEqual(0, log_mock.call_count)

    def test_call(self):
        '''Verify __call__ returns 0 when everything works.'''
        config = {'username', 'user',
                  'password', 'pass'}
        location = 'http://jenkins:8080/path/to/artifact'
        artifact_list = [
            'http://jenkins:8080/path/to/artifact.1',
            'http://jenkins:8080/path/to/artifact.2']
        jenkins_json = {'artifacts':
                        [{'relativePath': x} for x in artifact_list]}
        args = mock.Mock()
        args.datastore_config = None
        args.request_id = '1'
        args.job_name = 'lander_master'
        args.source_url = 'http://jenkins:8080/job/lander_master/1/'
        self.archiver.parse_arguments = lambda: args
        self.archiver.get_datastore_config = lambda x: config
        self.archiver.get_datastore = lambda x, y: 'datastore'
        self.archiver.get_url_contents = lambda x: json.dumps(jenkins_json)
        self.archiver.store_artifact = lambda x, y, z: location
        result = self.archiver()
        self.assertEqual(0, result)

    def test_call_datastore_config_error(self):
        '''Verify __call__ returns -1 on a datastore config loading error.'''
        args = mock.Mock()
        self.archiver.parse_arguments = lambda: args
        self.archiver.get_datastore_config = lambda x: None
        result = self.archiver()
        self.assertEqual(-1, result)

    def test_call_invalid_datastore(self):
        '''Verify __call__ returns -2 on a datastore creation error.'''
        config = {'username', 'user',
                  'password', 'pass'}
        args = mock.Mock()
        args.datastore_config = None
        self.archiver.parse_arguments = lambda: args
        self.archiver.get_datastore_config = lambda x: config
        self.archiver.get_datastore = lambda x, y: None
        result = self.archiver()
        self.assertEqual(-2, result)

    def test_call_store_artifact_failure(self):
        '''Verify __call__ returns -3 on a store artifact failure.'''
        config = {'username', 'user',
                  'password', 'pass'}
        args = mock.Mock()
        args.datastore_config = None
        args.request_id = '1'
        args.job_name = 'lander_master'
        args.source_url = 'http://jenkins:8080/job/lander_master/1/'
        self.archiver.parse_arguments = lambda: args
        self.archiver.get_datastore_config = lambda x: config
        datastore = mock.Mock()
        self.archiver.get_datastore = mock.Mock(return_value=datastore)
        result = self.archiver()
        self.assertEqual(-3, result)
